@@ -1,11 +1,13 @@
 use super::{
     concourse::{self},
-    config::Config,
+    config::*,
+    database::Database,
     repo::*,
     workspace::Workspace,
 };
 use anyhow::*;
 use clap::{clap_app, crate_version, App, ArgMatches};
+use std::path::Path;
 
 fn app() -> App<'static, 'static> {
     let app = clap_app!(cepler =>
@@ -13,6 +15,8 @@ fn app() -> App<'static, 'static> {
         (@setting VersionlessSubcommands)
         (@setting SubcommandRequiredElseHelp)
         (@arg CONFIG_FILE: -c --("config") env("CEPLER_CONF") default_value("cepler.yml") "Cepler config file")
+        (@arg GATES_FILE: -g --("gates") +takes_value env("CEPLER_GATES") "Cepler gate file")
+        (@arg GATES_BRANCH: --("gates-branch") +takes_value requires_all(&["GATES_FILE"]) env("GATES_BRANCH") "Branch to find the gate file")
         (@arg CLONE_DIR: --("clone") +takes_value requires_all(&["GIT_URL", "GIT_PRIVATE_KEY"]) "Clone the repository into <dir>. Pulls latest changes if already present.")
         (@arg GIT_URL: --("git-url") +takes_value env("GIT_URL") "Remote url for --clone option")
         (@arg GIT_PRIVATE_KEY: --("git-private-key") +takes_value env("GIT_PRIVATE_KEY") "Private key for --clone option")
@@ -25,6 +29,10 @@ fn app() -> App<'static, 'static> {
           (about: "List all files relevent to a given environment")
           (@arg ENVIRONMENT: -e --("environment") env("CEPLER_ENVIRONMENT") +required +takes_value "The cepler environment")
         )
+        (@subcommand latest =>
+          (about: "Return the commit hash of the lastest record")
+          (@arg ENVIRONMENT: -e --("environment") env("CEPLER_ENVIRONMENT") +required +takes_value "The cepler environment")
+        )
         (@subcommand record =>
           (about: "Record the state of an environment in the statefile")
           (@arg ENVIRONMENT: -e --("environment") env("CEPLER_ENVIRONMENT") +required +takes_value "The cepler environment")
@@ -71,6 +79,7 @@ pub fn run() -> Result<()> {
         let conf = GitConfig {
             url: matches.value_of("GIT_URL").unwrap().to_string(),
             branch: matches.value_of("GIT_BRANCH").unwrap().to_string(),
+            gates_branch: matches.value_of("GATES_BRANCH").map(|b| b.to_string()),
             private_key: matches.value_of("GIT_PRIVATE_KEY").unwrap().to_string(),
             dir: dir.to_string(),
         };
@@ -80,15 +89,33 @@ pub fn run() -> Result<()> {
             std::env::set_current_dir(dir)?;
         } else {
             std::env::set_current_dir(dir)?;
-            Repo::open()?.pull(conf)?;
+            Repo::open(None)?.pull(conf)?;
         }
     }
+
     match matches.subcommand() {
-        ("check", Some(sub_matches)) => check(sub_matches, &matches),
-        ("ls", Some(sub_matches)) => ls(sub_matches, &matches),
-        ("prepare", Some(sub_matches)) => prepare(sub_matches, conf_from_matches(&matches)?),
+        ("ls", Some(sub_matches)) => ls(
+            sub_matches,
+            conf_from_matches(&matches)?,
+            gates_from_matches(&matches)?,
+        ),
+        ("check", Some(sub_matches)) => check(
+            sub_matches,
+            conf_from_matches(&matches)?,
+            gates_from_matches(&matches)?,
+        ),
+        ("prepare", Some(sub_matches)) => prepare(
+            sub_matches,
+            conf_from_matches(&matches)?,
+            gates_from_matches(&matches)?,
+        ),
         ("reproduce", Some(sub_matches)) => reproduce(sub_matches, conf_from_matches(&matches)?),
-        ("record", Some(sub_matches)) => record(sub_matches, conf_from_matches(&matches)?),
+        ("record", Some(sub_matches)) => record(
+            sub_matches,
+            conf_from_matches(&matches)?,
+            gates_from_matches(&matches)?,
+        ),
+        ("latest", Some(sub_matches)) => latest(sub_matches, conf_from_matches(&matches)?),
         ("concourse", Some(sub_matches)) => match sub_matches.subcommand() {
             ("check", Some(_)) => concourse_check(),
             ("ci_in", Some(matches)) => concourse_in(matches),
@@ -99,16 +126,23 @@ pub fn run() -> Result<()> {
     }
 }
 
-fn check(matches: &ArgMatches, main_matches: &ArgMatches) -> Result<()> {
+fn check(
+    matches: &ArgMatches,
+    (config, config_path): (Config, String),
+    gates: Option<GatesConfig>,
+) -> Result<()> {
     let env = matches.value_of("ENVIRONMENT").unwrap();
-    let config = conf_from_matches(main_matches)?;
-    let ws = Workspace::new(config.1)?;
+    let gate = if let Some(gates) = gates {
+        gates.get_gate(env)?
+    } else {
+        None
+    };
+    let ws = Workspace::new(&config.scope, config_path)?;
     let env = config
-        .0
         .environments
         .get(env)
         .context(format!("Environment '{}' not found in config", env))?;
-    match ws.check(env)? {
+    match ws.check(env, gate)? {
         None => {
             println!("Nothing new to deploy");
             std::process::exit(2);
@@ -120,33 +154,49 @@ fn check(matches: &ArgMatches, main_matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn ls(matches: &ArgMatches, main_matches: &ArgMatches) -> Result<()> {
+fn ls(
+    matches: &ArgMatches,
+    (config, config_path): (Config, String),
+    gates: Option<GatesConfig>,
+) -> Result<()> {
     let env = matches.value_of("ENVIRONMENT").unwrap();
-    let config = conf_from_matches(main_matches)?;
-    let ws = Workspace::new(config.1)?;
+    let gate = if let Some(gates) = gates {
+        gates.get_gate(env)?
+    } else {
+        None
+    };
+    let ws = Workspace::new(&config.scope, config_path)?;
     let env = config
-        .0
         .environments
         .get(env)
         .context(format!("Environment '{}' not found in config", env))?;
-    for path in ws.ls(env)? {
+    for path in ws.ls(env, gate)? {
         println!("{}", path);
     }
     Ok(())
 }
-fn prepare(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
+fn prepare(
+    matches: &ArgMatches,
+    config: (Config, String),
+    gates: Option<GatesConfig>,
+) -> Result<()> {
     let env = matches.value_of("ENVIRONMENT").unwrap();
     let force_clean: bool = matches.is_present("FORCE_CLEAN");
     if force_clean {
         println!("WARNING removing all non-cepler specified files");
     }
+    let gate = if let Some(gates) = gates {
+        gates.get_gate(env)?
+    } else {
+        None
+    };
     let env = config
         .0
         .environments
         .get(env)
         .context(format!("Environment '{}' not found in config", env))?;
-    let ws = Workspace::new(config.1)?;
-    ws.prepare(env, force_clean)?;
+    let ws = Workspace::new(&config.0.scope, config.1)?;
+    ws.prepare(env, gate, force_clean)?;
     Ok(())
 }
 fn reproduce(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
@@ -160,13 +210,22 @@ fn reproduce(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
         .environments
         .get(env)
         .context(format!("Environment '{}' not found in config", env))?;
-    let ws = Workspace::new(config.1)?;
+    let ws = Workspace::new(&config.0.scope, config.1)?;
     ws.reproduce(env, force_clean)?;
     Ok(())
 }
 
-fn record(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
+fn record(
+    matches: &ArgMatches,
+    config: (Config, String),
+    gates: Option<GatesConfig>,
+) -> Result<()> {
     let env = matches.value_of("ENVIRONMENT").unwrap();
+    let gate = if let Some(gates) = gates {
+        gates.get_gate(env)?
+    } else {
+        None
+    };
     let commit = !matches.is_present("NO_COMMIT");
     let reset = matches.is_present("RESET_HEAD");
     let push = matches.is_present("PUSH");
@@ -174,6 +233,7 @@ fn record(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
         Some(GitConfig {
             url: matches.value_of("GIT_URL").unwrap().to_string(),
             branch: matches.value_of("GIT_BRANCH").unwrap().to_string(),
+            gates_branch: None,
             private_key: matches.value_of("GIT_PRIVATE_KEY").unwrap().to_string(),
             dir: String::new(),
         })
@@ -185,8 +245,20 @@ fn record(matches: &ArgMatches, config: (Config, String)) -> Result<()> {
         .environments
         .get(env)
         .context(format!("Environment '{}' not found in config", env))?;
-    let mut ws = Workspace::new(config.1)?;
-    ws.record_env(env, commit, reset, git_config)?;
+    let mut ws = Workspace::new(&config.0.scope, config.1)?;
+    ws.record_env(env, gate, commit, reset, git_config)?;
+    Ok(())
+}
+
+fn latest(matches: &ArgMatches, (config, config_file): (Config, String)) -> Result<()> {
+    let env = matches.value_of("ENVIRONMENT").unwrap();
+    let db = Database::open(&config.scope, &config_file)?;
+    if let Some(env) = db.get_current_state(env) {
+        println!("{}", env.head_commit.clone().inner());
+    } else {
+        eprintln!("Environment '{}' not deployed!", env);
+        std::process::exit(1);
+    }
     Ok(())
 }
 
@@ -208,3 +280,23 @@ fn conf_from_matches(matches: &ArgMatches) -> Result<(Config, String)> {
     let file_name = matches.value_of("CONFIG_FILE").unwrap();
     Ok((Config::from_file(file_name)?, file_name.to_string()))
 }
+
+#[allow(clippy::redundant_closure)]
+fn gates_from_matches(matches: &ArgMatches) -> Result<Option<GatesConfig>> {
+    let file_name = matches.value_of("GATES_FILE");
+    if let Some(branch) = matches.value_of("GATES_BRANCH") {
+        match Repo::open(None)?.get_file_from_branch(
+            branch,
+            Path::new(file_name.unwrap()),
+            |bytes| GatesConfig::from_reader(bytes),
+        ) {
+            Ok(Some(config)) => Ok(Some(config)),
+            Ok(_) => Err(anyhow!("Couldn't find gates file in branch")),
+            err => err,
+        }
+    } else if let Some(f) = file_name {
+        Ok(Some(GatesConfig::from_file(f)?))
+    } else {
+        Ok(None)
+    }
+}
